import { jsxs as t, jsx as e } from "react/jsx-runtime";

import { useState as n, useLayoutEffect as o, useMemo as r, useCallback as i, useEffect as s, useRef as l } from "react";

import { VirtualScrollerEvent as c, VirtualScroller as f } from "@af-utils/virtual-core";

import { useSyncExternalStore as u } from "use-sync-external-store/shim";

/**
 * @public
 * Map current visible items
 * @param model - {@link @af-utils/virtual-core#VirtualScroller} instance
 * @param cb - callback, returning ONLY ONE react element
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */ function a(t, e) {
    const n = [];
    for (let o = t.from, r = t.to; r > o; o++) n.push(e(o));
    return n;
}

/**
 * @public
 * Map current visible items and provide offset for each item.
 *
 * @remarks
 * Used in scenarios, when each item is absolutely positioned
 *
 * @param model - {@link @af-utils/virtual-core#VirtualScroller} instance
 * @param cb - callback, returning ONLY ONE react element
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */ function h(t, e) {
    const n = [];
    for (let o = t.from, r = t.to, i = t.getOffset(o); r > o; o++, i += t.getSize(o)) n.push(e(o, i));
    return n;
}

const d = {
    position: "relative",
    overflow: "hidden",
    contain: "strict",
    // otherwise may go above sticky header
    zIndex: -1
}, m = {
    ...d,
    height: "100%"
}, p = {
    ...d,
    width: "100%"
}, S = {
    position: "absolute",
    contain: "strict",
    overflow: "hidden",
    top: 0,
    left: 0
}, g = {
    ...S,
    display: "flex",
    height: "100%"
}, v = {
    ...S,
    display: "block",
    width: "100%"
};

/**
 * @public
 * React hook.
 * Optimal CSS markup for virtual scroll is not intuitive.
 * Use this hook to avoid unneeded boilerplate.
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */
function y(t) {
    const [e, r] = n(null), [i, s] = n(null);
    return o((() => {
        if (t && e && i) {
            const n = () => {
                e.style[t.horizontal ? "width" : "height"] = t.scrollSize + "px";
            }, o = () => {
                const e = t.getOffset(t.from), n = t.getOffset(t.to);
                i.style.transform = `translate${t.horizontal ? "X" : "Y"}(${e}px)`, i.style[t.horizontal ? "width" : "height"] = n - e + "px";
            }, r = t.on(n, [ c.SCROLL_SIZE ]), s = t.on(o, [ c.RANGE, c.SIZES ]);
            return Object.assign(e.style, t.horizontal ? m : p), Object.assign(i.style, t.horizontal ? g : v), 
            n(), o(), () => {
                r(), s();
            };
        }
    }), [ t, e, i ]), [ r, s ];
}

function x(t) {
    switch (t) {
      case c.RANGE:
        return this.to ** 2 + this.from;

 // szudzik pair
              case c.SCROLL_SIZE:
        return this.scrollSize;

      default:
        return this.sizesHash;
    }
}

/**
 * @public
 * React hook.
 * Rerenders component when one of {@link @af-utils/virtual-core#(VirtualScrollerEvent:variable)} gets emitted.
 * Usually {@link Subscription} is a better alternative
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */ function z(t, e) {
    const [n, o] = r((() => [ n => t.on(n, e), () => e.map(x, t).join() ]), [ t, e ]);
    u(n, o, o);
}

/**
 * @public
 * React component.
 * Rerenders only on certain {@link @af-utils/virtual-core#(VirtualScrollerEvent:variable)}.
 * Allows to optimize performance.
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */ function w(t) {
    return z(t.model, t.events), t.children();
}

/**
 * @public
 * React component.
 * Small abstraction, which in 90% cases allows to avoid hook boilerplate.
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */ function E(n) {
    const {model: o, children: r, itemData: s, component: l, header: f = null, footer: u = null, getKey: h = (t => t), tabIndex: d = -1, style: m, ...p} = n, S = l || "div", [g, v] = y(o);
    return t(S, {
        ...p,
        style: {
            overflow: "auto",
            contain: "strict",
            ...m
        },
        ref: i((t => o.setScroller(t)), [ o ]),
        tabIndex: d,
        children: [ f, e("div", {
            ref: g,
            children: e("div", {
                ref: v,
                children: e(w, {
                    model: o,
                    events: [ c.RANGE ],
                    children: () => a(o, (t => e(r, {
                        model: o,
                        i: t,
                        data: s
                    }, h(t, s))))
                })
            })
        }), u ]
    });
}

const I = process.env.__IS_SERVER__ ? s : o;

/**
 * @public
 * React hook.
 * Synchronizes scroller with model.
 *
 * @remarks
 * Should be used in window-scroll cases, otherwise `ref={el => model.setScroller( el )}` is preferrable.
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */ function O(t, e) {
    I((() => (t.setScroller(e), () => t.setScroller(null))), [ t, e ]);
}

/**
 * @public
 * React hook.
 * Allows to subscribe to {@link @af-utils/virtual-core#(VirtualScrollerEvent:variable)} without unnecessary rerenders.
 *
 * @remarks
 * For example can be used in load-on-demand.
 *
 * @param model - {@link @af-utils/virtual-core#VirtualScroller} instance
 * @param events - array of {@link @af-utils/virtual-core#(VirtualScrollerEvent:variable)} to subscribe
 * @param callBack - callback to be called
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */ function R(t, e, n) {
    s((() => {
        if (n) return n(), t.on(n, e);
    }), [ t, n, e ]);
}

/**
 * @public
 * React hook.
 * Creates and stores one {@link @af-utils/virtual-core#VirtualScroller} instance.
 * It does not get recreated during component lifecycle.
 * Normally {@link useVirtual} should be used.
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */ function _(t) {
    return l().current ||= new f(t);
}

/**
 * @public
 * React hook.
 * Calls {@link useVirtualModel} and synchronizes it with props
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */ function b(t) {
    const e = _(t);
    return I((() => e.set(t))), e;
}

export { E as List, w as Subscription, a as mapVisibleRange, h as mapVisibleRangeWithOffset, z as useComponentSubscription, O as useScroller, R as useSubscription, y as useSyncedStyles, b as useVirtual, _ as useVirtualModel };
//# sourceMappingURL=index.js.map
